@@ -1,5 +1,5 @@
 #
-# $Id: Ingperl.pm,v 2.100 1997/09/10 08:00:41 ht000 Exp $
+# $Id: Ingperl.pm,v 1.6 1996/12/02 11:56:06 ht Exp $
 #
 # Ingperl emulation interface for DBD::Ingres
 #
@@ -17,7 +17,7 @@ use DBI 0.73;
 use Exporter;
 use Carp;
 
-$VERSION = substr(q$Revision: 2.100 $, 10);
+$VERSION = substr(q$Revision: 1.6 $, 10);
 
 @ISA = qw(Exporter);
 
@@ -30,22 +30,26 @@ $VERSION = substr(q$Revision: 2.100 $, 10);
     &sql_eval_row1 &sql_eval_col1
 );
 
-use strict;
-use vars (qw[$sql_drh $sql_dbh $sql_sth $sql_debug]);
+$debug    = 0 unless defined $debug;
+$debugdbi = 0;
 
-$sql_debug    = 0 unless defined $sql_debug;
-
-if ($sql_debug){
+if ($debugdbi){
     my $sw = DBI->internal;
+    $sw->debug($debugdbi);
     print "Switch: $sw->{Attribution}, $sw->{Version}\n";
+    $sw->{DebugDispatch} = $debugdbi;
 }
 
+
+use strict;
+use vars (qw[$sql_drh $sql_dbh $sql_sth $debug]);
+
 # Install Driver
 $sql_drh = DBI->install_driver('Ingres');
 if ($sql_drh) {
-    print "DBD::Ingres driver installed as $sql_drh\n" if $sql_debug;
-    $sql_drh->{Warn}       = 0;
-    $sql_drh->{CompatMode} = 1;
+	print "DBD::Ingres driver installed as $sql_drh\n" if $debug;
+	$sql_drh->debug($debug);
+	$sql_drh->{Warn}       = 0;
 }
 
 
@@ -60,76 +64,66 @@ if ($sql_drh) {
 sub sql_exec {
     my($statement) = @_;
     # decide what this is...
-    warn "sql_exec('$statement')\n" if $sql_debug;
+    warn "sql_exec('$statement')\n" if $Ingperl::debug;
     if ($statement =~ /^\s*connect\b/i) {
         # connect to the database;
-        croak "Already connected to database, at" if $sql_dbh;
+        croak "Already connected to database" if $sql_dbh;
         my($database, $user, $option);
         # this contain the database name and possibly
         # a username
         # find database
-        ($database) = $statement =~ m!connect\s+([\w:/]+)!i;
+        ($database) = $statement =~ /connect\s+([\w:]+)/;
         my $rest = $';  #possibly contains username... and other options
-        if ($rest =~ /identified\s+by\s+(\w+)/i) {
+        if ($rest =~ /identified\s+by\s+(\w+)/) {
             $user = $1;
-            $option = "$` $'"; # every thing else..
+            $option = "$`$'"; # every thing else..
         } elsif ($rest =~ /-u(\w+)/) {
             $user = $1;
-            $option = "$` $'"; # every thing else..
+            $option = "$`$'"; # every thing else..
         } else {
             $user = ""; # noone;
             $option = $rest
         }
         warn "Ingperl connecting to database '$database' as user '$user'\n"
-            if $sql_debug;
+            if $Ingperl::debug;
 	$option =~ s/^\s+//;
-        $sql_dbh = $Ingperl::sql_drh->connect("$database;$option", $user);
-    } else {
-        croak "Ingperl: Not connected to database, at" unless $sql_dbh;
-
-	if ($statement =~ /^\s*disconnect\b/i) {
-            $sql_dbh->disconnect();
-            undef $sql_dbh;
-    	}
-    	elsif ($statement =~ /^\s*commit\b/i) {
-            $sql_dbh->commit();
-        }
-	elsif ($statement =~ /^\s*rollback\b/i) {
-            $sql_dbh->rollback();
-        }
-        else {
-            # This is something else. Just execute the statement
-           $sql_dbh->do($statement);
-        }
+        $sql_dbh = $Ingperl::sql_drh->connect($database, $user, $option);
+    }
+    elsif ($statement =~ /^\s*disconnect\b/i) {
+        croak "Ingperl: Not connected to database" unless $sql_dbh;
+        $sql_dbh->disconnect();
+        undef $sql_dbh;
+    }
+    elsif ($statement =~ /^\s*commit\b/i) {
+        croak "Ingperl: Not connected to database" unless $sql_dbh;
+        $sql_dbh->commit();
+    }
+    elsif ($statement =~ /^\s*rollback\b/i) {
+        croak "Ingperl: Not connected to database" unless $sql_dbh;
+        $sql_dbh->rollback();
+    }
+    else {
+        # This is something else. Just execute the statement
+        croak "Ingperl: Not connected to database" unless $sql_dbh;
+        $sql_dbh->do($statement);
     }
 }
 
 sub sql_close {
-    if ($sql_sth) {
-        $sql_sth->finish;
-        undef $sql_sth;
-        1;
-    } else {
-        carp "Ingperl: close with no open cursor, at"
-            if $sql_drh->{Warn};
-        1;
-    }
+	if ($sql_sth) {
+	    $sql_sth->finish;
+	    undef $sql_sth;
+	    1;
+	} else {
+	    carp "Ingperl: close with no open cursor" unless $sql_sth;
+	}
 }
 
 sub sql_fetch {
-    croak "Ingperl: No active cursor, at" unless $sql_sth;
+    croak "Ingperl: No active cursor\n" unless $sql_sth;
     my(@row) = $sql_sth->fetchrow();
-    unless (@row) {
-	&sql_close();
-	return wantarray ? () : undef;
-    }
-    if (wantarray) {
-        return @row;
-    } else { # wants a scalar
-        carp "Multi-column row retrieved in scalar context, at"
-            if $sql_sth->{Warn};
-        return $sql_sth->{CompatMode} ? $row[0] : @row;
-    }
+    &sql_close() unless @row;
+    @row;
 }
 
 sub sql {
@@ -139,11 +133,9 @@ sub sql {
     }
     elsif ($statement =~ /^\s*select\b/i) {
         if ($sql_sth) {
-            warn "IngPerl: Select while another select active - closing".
-            	" previous select, at"
-                    if $sql_debug or $sql_sth->{Warn};
+            warn "IngPerl: Select while another select active\n".
+                 "         Closing previous select: $sql_sth\n" if $Ingperl::debug;
             $sql_sth->finish();
-            undef $sql_sth;
         }
         $sql_sth = $sql_dbh->prepare($statement) or return undef;
         $sql_sth->execute() or return undef;
@@ -161,11 +153,11 @@ sub sql {
 # @names = &sql_names;
 #
 
-sub sql_types       { $sql_sth ? @{$sql_sth->{'ing_types'}}   : undef; }
-sub sql_ingtypes    { $sql_sth ? @{$sql_sth->{'ing_ingtypes'}}: undef; }
-sub sql_lengths     { $sql_sth ? @{$sql_sth->{'ing_lengths'}} : undef; }
-sub sql_nullable    { $sql_sth ? @{$sql_sth->{'NULLABLE'}}    : undef; }
-sub sql_names       { $sql_sth ? @{$sql_sth->{'NAME'}}        : undef; }
+sub sql_types       { $sql_sth ? @{$sql_sth->{'TYPE'}}     : undef; }
+sub sql_ingtypes    { $sql_sth ? @{$sql_sth->{'SqlType'}}  : undef; }
+sub sql_lengths     { $sql_sth ? @{$sql_sth->{'SqlLen'}}   : undef; }
+sub sql_nullable    { $sql_sth ? @{$sql_sth->{'NULLABLE'}} : undef; }
+sub sql_names       { $sql_sth ? @{$sql_sth->{'NAME'}}     : undef; }
 
 # *----------------------------------------
 #
@@ -174,33 +166,29 @@ tie $Ingperl::sql_version,   'Ingperl::var', 'version';
 *sql_error = \$DBD::Ingres::errstr;
 *sql_sqlcode = \$DBD::Ingres::err;
 *sql_rowcount = \$DBI::rows;
-tie $Ingperl::sql_readonly,   'Ingperl::var', 'readonly';
-tie $Ingperl::sql_showerrors, 'Ingperl::var', 'showerror';
+$Ingperl::sql_readonly = 1;
+$Ingperl::sql_showerrors = 0;
+$Ingperl::sql_debug = 0;
 
 # *----------------------------------------
 #
 # Library function to execute a select and return first row
 sub sql_eval_row1{
-	my $sth = $sql_dbh->prepare(@_);
-	return undef unless $sth;
-	$sth->execute or return undef;
-	my(@row) = $sth->fetchrow;	# fetch one row
-	$sth->finish;			# close the cursor
-	undef $sth;
+	&sql(@_);
+	return undef if $::sql_error;
+	my(@row) = &sql_fetch;	# fetch one row
+	&sql_close;					# close the cursor
 	@row;
 }
 
 # Library function to execute a select and return first col
 sub sql_eval_col1{
-	my $sth = $sql_dbh->prepare(@_);
-	return undef unless $sth;
-	$sth->execute or return undef;
-        my ($row, @col);
-	while ($row = $sth->fetch){
-		push(@col, $row->[0]);
+	&sql(@_);
+	return undef if $::sql_error;
+	my(@row, @col);
+	while (@row = &sql_fetch){
+		push(@col, $row[0]);
 	}
-	$sth->finish;			# close the cursor
-	undef $sth;
 	@col;
 }
 
@@ -224,24 +212,14 @@ sub FETCH {
         "$Ingperl::sql_drh->{'Attribution'}\n" .
         $sw->{'Attribution'}. ", ".
         "version " . $sw->{'Version'}. "\n\n";
-    } elsif ($$self eq "readonly") {
-    	1;   # Not implemented (yet)
-    } elsif ($$self eq "showerror") {
-    	$Ingperl::sql_dbh->{printerror};
-    } else {
+    }
+    else {
         carp "unknown special variable $$self";
     }
 }
 
 sub STORE {
-    my ($self, $value) = shift;
-    confess "wrong type" unless ref $self;
-    croak "too many arguments" if @_;
-    if ($$self eq "showerror") {
-    	$Ingperl::sql_dbh->{printerror} = $value;
-    } else {
-        carp "Can't modify ${$self} special variable, at"
-    }
+    carp "Can't modify ${$_[0]} special variable"
 }
 
 1;
@@ -287,10 +265,9 @@ All functions return false or undefined (in the Perl sense)
 to indicate failure.
 
 The text in this document is largely unchanged from the original Perl4
-ingperl documentation written by Tim Bunce (timbo@ig.co.uk).
-
-Any comments specific to the DBD::Ingres Ingperl emulation are prefixed
-by B<DBD:>.
+ingperl documentation written by Tim Bunce (timbo@ig.co.uk).  Any
+comments specific to the DBD::Ingres Ingperl emulation are prefixed by
+B<DBD:>.
 
 =head2 IngPerl Functions
 
@@ -319,18 +296,6 @@ For example:
 
 Returns true else undef on error.
 
-B<DBD:> Note that the options B<must> be given in the order
-C<database-name username other-options> otherwise the check for username
-wil fail. It is a rather simple scan of the option-string. Further
-improvements are welcome.
-
-B<DBD:> The connect must be given as "C<connect> I<database-name>
-C<identified by> I<username> I<SQL-options>" or "C<connect>
-I<database-name>C< -u>I<username> I<SQL-options>".
-
-B<DBD:>There is (yet) no way of passing a password to the connect call.
-Suggestion: add "password=I<password>" just after I<username>.
-
 =item disconnect from a database:
 
     &sql("disconnect");
@@ -339,19 +304,12 @@ Note that ingperl will complain if a transaction is active.
 
 You should &sql_exec 'commit' or 'rollback' before disconnect.
 
-B<DBD:> The warning on disconnect has another wording now:
-	Ingres: You should commit or rollback before disconnect.
-        Ingres: Any outstanding changes have been rolledback.
-B<DBD:> Please note the C<rollback>.
-
 Returns true else undef on error (unlikely!).
 
-Note that an ingres bug means that $sql_error will contain an error
-message (E_LQ002E query issued outside of a session) even though the
-disconnect worked ok.
-
-B<DBD:> I<Must check if this is still the case.> Don't think so
-though...
+Note that an ingres bug means that $sql_error will contain an
+error message (E_LQ002E query issued outside of a session) even
+though the disconnect worked ok.
+B<DBD:> I<Must check if this is still the case...>
 
 =item prepare a statement:
 
@@ -385,7 +343,7 @@ hands off all non-select statements to C<&sql_exec>.
 
 =back
 
-=item * sql_exec
+item * sql_exec
 
     &sql_exec('...');
 
@@ -429,46 +387,24 @@ For example:
 
 Null values are returned as undef elements of the array.
 
-B<DBD:> C<&sql_fetch> can also be expressed as either C<&sql("fetch")>
-or C<&sql_exec("fetch")> - to cater for Ingperl 1.0 scripts!
-
-B<DBD:> C<&sql_fetch> will call C<&sql_close> when the last row of data
-has been fetched. This has been the way it was supposed to be...
-
-B<DBD:> C<&sql_fetch> will die with the error C<Ingperl: No active
-cursor> if an error has occured in the C<&sql(select..)>-statement.
-
-B<DBD:> C<$scalar = &sql_fetch> returns the first column of data if
-C<$sql_sth-E<gt>{CompatMode}> is set; this is the default mode for
-Ingperl and is the expected behaviour for Perl4. In Perl5 (and with 
-C<$sql_sth-E<gt>{CompatMode}> unset) the number of columns will be
-returned. The warning C<Multi-column row retrieved in scalar context>
-is given if C<$sql_sth-E<gt>{Warn}> is true.
-
-B<DBD:> Text columns are returned with trailing blanks if
-C<$sql_sth-E<gt>{CompatMode}> is set. Otherwise the trailings
-blanks are stripped.
-The default for C<Ingperl> is to have C<$sql_sth-E<gt>{CompatMode}>
-set.
+B<DBD:> C<&sql_fetch> can also be expressed as either
+C<&sql("fetch")> or C<&sql_exec("fetch")> - to cater for
+Ingperl 1.0 scripts!
 
 =item * sql_close
 
     &sql_close;
 
-This function needs to be called B<only> if you do not use C<&sql_fetch>
-to fetch B<all> the records B<and> you wish to close the cursor as soon as
-possible (to release locks etc). Otherwise ignore it. Always returns
-true.
-
-B<DBD:> If C<$sql_sth-E<gt>{Warn}> is false the warning C<Ingperl: close
-with no open cursor> will be given whenever a closed cursor is reclosed.
-The default behaviour is to omit the warning.
+This function needs to be called *only* if you do not use
+C<&sql_fetch> to fetch *all* the records *and* you wish to close
+the cursor as soon as possible (to release locks etc).
+Otherwise ignore it. Always returns true.
 
 =back
 
-IngPerl Functions to describe the currently prepared statement. These
-functions all return an array with one element for each field in the
-query result.
+IngPerl Functions to describe the currently prepared statement.
+These functions all return an array with one element for each
+field in the query result.
 
 =over 4
 
@@ -477,10 +413,7 @@ query result.
     @types = &sql_types;
 
 Returns a list of sprintf type letters to indicate the generic
-type of each field:
-      d - int
-      f - float
-      s - string
+type if each field: 'd' (int), 'f' (float), or 's' (string).
 
 =item * sql_ingtypes
 
@@ -499,23 +432,16 @@ Returns a list of specific ingres type numbers:
     @lengths = &sql_lengths;
 
 Returns a list if ingres data type lengths.
-
 For strings the length is the maximum width of the field.
-
-For numbers it is the number of bytes used to store the binary
-representation of the value, 1, 2, 4 or 8.
-
-For date and money fields it is 0.
-
-B<DBD:> This was not documented in the Ingperl documentation, but is, as
-far as I can discover, how it used to work.
+For numbers it is the number of bytes used to store the
+binary representation of the value, 1, 2, 4 or 8.
 
 =item * sql_nullable
 
     @nullable = &sql_nullable;
 
-Returns a list of boolean values (0 or 1's). A 1 indicates that the
-corresponding field may return a null value.
+Returns a list of boolean values (0 or 1's). A 1 indicates
+that the corresponding field may return a null value.
 
 =item * sql_names
 
@@ -543,9 +469,9 @@ Ingperl version and the DBD::Ingres version.
 
 =item * $sql_error (read only)
 
-Contains the error message text of the current Ingres error.
+Contains the error message text of the current ingres error.
 
-Is empty if last statement succeeded.
+Is empty if last statement succedded.
 
 For example:
     print "$sql_error\n" if $sql_error;
@@ -555,45 +481,45 @@ For example:
 The current value of sqlda.sqlcode. Only of interest in more
 sophisticated applications.
 
-Typically 0, <0 on error, 100=no more rows, 700=message, 710=dbevent.
+Typically 0, <0 on error,
+100=no more rows, 700=message, 710=dbevent.
 
 =item * $sql_rowcount (read only)
 
-After a successful Insert, Delete, Update, Select, Modify, Create Index,
-Create Table As Select or Copy this variable holds the number of rows
-affected.
+After a successful Insert, Delete, Update, Select, Modify,
+Create Index, Create Table As Select or Copy this variable
+holds the number of rows affected.
 
 =item * $sql_readonly (default 1)
 
-If true then prepared sql statements are given read only cursors this is
-generally a considerable performance gain.
+If true then prepared sql statements are given read only cursors
+this is generally a considerable performance gain.
 
 B<DBD:> Not implemented. All cursors are readonly - there is no way to
 modify the value of a cursor element, therefore no reason not to make
-the cursors readonly. The value of this variable was ignored already in
-Ingperl 2.0.
+the cursors readonly. The value of this variable was ignored already
+in Ingperl 2.0.
 
 =item * $sql_showerrors (default 0)
 
 If true then ingres error and warning messages are printed by
 ingperl as they happen. Very useful for testing.
 
-B<DBD:> Same as $sql_dbh->{printerror}
+B<DBD:> Not yet implemented. (Does anybody need it?)
 
 =item * $sql_debug (default 0)
 
 If ingperl has been compiled with -DINGPERL_DEBUG then setting this
 variable true will enable debugging messages from ingperl internals.
 
-B<DBD:> Setting this variable will enable debugging from the Ingperl
-emulation layer. Setting the variable C<$DBI::dbi_debug> enables debug output
-from the C<DBI> and C<DBD> layers (the value 3 will result in large
-amounts of output including internal debug of C<DBD::Ingres>).
+B<DBD:> Not implemented. Setting the variable C<$debugdbi> to 3
+or greater results in debug information from DBI and DBD::Ingres
 
 =item * $sql_drh
 
 B<DBD:> This variable is the DBI-internal driver handle for the
-DBD::Ingres driver. This is rarely used!
+DBD::Ingres driver. It is of little or no use at present especially as
+there is no provision for multiple connects yet).
 
 =item * $sql_dbh
 
@@ -607,7 +533,7 @@ B<DBD:> This is the DBI statement handle for the current SELECT-statement
 
 =back
 
-=head2 IngPerl Library Functions
+IngPerl Library Functions
 
 =over 4
 
@@ -617,50 +543,46 @@ B<DBD:> This is the DBI statement handle for the current SELECT-statement
 
 Execute a select statement and return the first row.
 
-B<DBD:> This is executed in a separate cursor and can therefore be
-executed while a &sql_fetch-loop is in progres.
-
 =item * sql_eval_col1
 
     @col1 = &sql_eval_col1('select ...');
 
 Execute a select statement and return the first column.
 
-B<DBD:> As &sql_eval_col1 this is executed in a separate cursor.
-
 =head1 NOTES
 
 The DBD::Ingres module has been modelled closely on Tim Bunce's
-DBD::Oracle module and warnings that apply to DBD::Oracle and the
-Oraperl emulation interface may also apply to the Ingperl emulation
-interface.
+DBD::Oracle module and warnings that apply to DBD::Oracle and
+the Oraperl emulation interface may also apply to the Ingperl
+emulation interface.
 
 Your mileage may vary.
 
 =head1 WARNINGS
 
-IngPerl comes with no warranty - it works for me - it may not work for
-you. If it trashes your database I am not responsible!
 
-This file should be included in all applications using ingperl in order
-to help ensure that scripts will remain compatible with new releases of
-ingperl.
+IngPerl comes with no warranty - it works for me - it may not
+work for you. If it trashes your database I am not responsible!
+
+This file should be included in all applications using ingperl
+in order to help ensure that scripts will remain compatible with
+new releases of ingperl.
 
 B<DBD:> The following warning is taken (almost) verbatim from the
 oraperl emulation module, but is also valid for Ingres.
 
 The Ingperl emulation software shares no code with the original ingperl.
-It is built on top the the new Perl5 DBI and DBD::Ingres modules. These
-modules are still evolving. (One of the goals of the Ingperl emulation
-software is to allow useful work to be done with the DBI and DBD::Ingres
-modules whilst insulation users from the ongoing changes in their
-interfaces.)
+It is built on top the the new Perl5 DBI and DBD::Ingres
+modules. These modules are still evolving. (One of the goals of
+the Ingperl emulation software is to allow useful work to be done
+with the DBI and DBD::Ingres modules whilst insulation users from
+the ongoing changes in their interfaces.)
 
 It is quite possible, indeed probable, that some differences in
 behaviour will exist. This should be confined to error handling.
 
-B<All> differences in behaviour which are not documented here should be
-reported to ht@datani.dk and CC'd to dbi-users@fugue.com.
+B<All> differences in behaviour which are not documented here should
+be reported to ht@datani.dk and CC'd to dbi-users@fugue.com.
 
 
 =head1 SEE ALSO
